import { useEffect } from "react";
import { ApiRespons, FeilType, fetcherPost, ikkeRevalidate } from "./api";
import { GCP_MEDIATOR_URL } from "../utils/baseUrl";
import { Produksjonsdetaljer } from "../kartSide/types/produksjonsdetaljer";
import { Materiale, MaterialKategori, MaterialType } from "../kartSide/types/materiale";
import { tilEnkelTimestampStr } from "../utils/tid";
import { Dayjs } from "dayjs";
import { MaterialOgAktivitetsNavn } from "./context/ProduksjonsfilterAtoms";
import { Point } from "geojson";
import { atom, useSetAtom } from "jotai";
import useSWR from "swr";

export interface ProduksjonsdetaljerDto {
    produksjonsdataRestRec: number[];
    kjoeretoyid?: string;
    datoLogget: string;
    hastighetkmhavg: number;
    erplognede?: boolean;
    ertoerrstoffsprederpaa?: boolean;
    ervaatstoffsprederpaa?: boolean;
    sprederbreddetoerrstoffcm?: number;
    sprederbreddevaatstoffcm?: number;
    doseringtoerrstoffgm2?: number;
    doseringvaatstoffmlm2?: number;
    erhoevelskjaernede?: boolean;
    erunderliggendeskjaernede?: boolean;
    ersideplogibruk?: boolean;
    materiale?: string;
    eksternReferanse?: string;
    minutterSiden: number;
    startLokasjon: LokasjonOgRecordId;
    sluttLokasjon: LokasjonOgRecordId;
    vegsystemreferansekortform?: string;
    rodeId: string[];
}

interface LokasjonOgRecordId {
    recordId: number;
    punkt: Point;
}

export const produksjonsdetaljerAtom = atom<ApiRespons<Produksjonsdetaljer[]> | undefined>(
    undefined,
);

export const useHentProduksjondetaljer = (målepunkter: number[], tidspunkt: Dayjs) => {
    const setProduksjonsdetaljer = useSetAtom(produksjonsdetaljerAtom);
    const key = `${GCP_MEDIATOR_URL}/aggregerte-produksjonsdetaljer?tidspunkt=${tilEnkelTimestampStr(
        tidspunkt,
    )}&målepunkter=${JSON.stringify(målepunkter)}`;

    const customFetcher = () => {
        return hentProduksjonsdetaljer(målepunkter, tidspunkt);
    };
    const { data } = useSWR(key, customFetcher, ikkeRevalidate);

    useEffect(() => {
        setProduksjonsdetaljer(data);
    }, [data]);
};

export const hentProduksjonsdetaljer = async (
    målepunkter: number[],
    tidspunkt: Dayjs,
): Promise<ApiRespons<Produksjonsdetaljer[]>> => {
    const url = `${GCP_MEDIATOR_URL}/aggregerte-produksjonsdetaljer?tidspunkt=${tilEnkelTimestampStr(
        tidspunkt,
    )}`;
    const body = { målepunkter: målepunkter };
    return fetcherPost<ProduksjonsdetaljerDto[]>(
        url,
        FeilType.KUNNE_IKKE_HENTE_PRODUKSJONSDETALJER,
        JSON.stringify(body),
    ).then((response) =>
        response.suksess
            ? {
                  ...response,
                  data: response.data.map((dto) => dtoTilProduksjonsdetaljer(dto)),
              }
            : response,
    );
};

export const dtoTilProduksjonsdetaljer = (dto: ProduksjonsdetaljerDto): Produksjonsdetaljer => ({
    produksjonsdataRestRec: dto.produksjonsdataRestRec,
    kjøretøyId: dto.kjoeretoyid,
    datoLogget: dto.datoLogget,
    hastighetKmHAvg: dto.hastighetkmhavg,
    materiale: dtoTilMateriale(dto),
    erPlogNede: dto.erplognede ?? false,
    erTørrstoffsprederPå: dto.ertoerrstoffsprederpaa,
    erVåtstoffsprederPå: dto.ervaatstoffsprederpaa,
    eksternReferanse: dto.eksternReferanse,
    erUnderliggendeSkjærNede: dto.erunderliggendeskjaernede ?? false,
    erHøvelskjærNede: dto.erhoevelskjaernede ?? false,
    erSideplogIBruk: dto.ersideplogibruk ?? false,
    sprederbreddetoerrstoffcm: dto.sprederbreddetoerrstoffcm ?? 0,
    sprederbreddevaatstoffcm: dto.sprederbreddevaatstoffcm ?? 0,
    doseringtoerrstoffgm2: dto.doseringtoerrstoffgm2 ?? 0,
    doseringvaatstoffmlm2: dto.doseringvaatstoffmlm2 ?? 0,
    minutterSiden: dto.minutterSiden,
    startLokasjon: dto.startLokasjon,
    sluttLokasjon: dto.sluttLokasjon,
    vegsystemreferansekortform: dto.vegsystemreferansekortform,
    rodeId: dto.rodeId,
});

const dtoTilMateriale = (dto: ProduksjonsdetaljerDto): Materiale | undefined => {
    if (dto.materiale === "Tørrsand") {
        return {
            navn: MaterialOgAktivitetsNavn.Tørrsand,
            type: MaterialType.Tørrsand,
            kategori: MaterialKategori.Sand,
        };
    }
    if (dto.materiale === "Sand med saltinnblanding") {
        return {
            navn: MaterialOgAktivitetsNavn.SandMedSaltInnblanding,
            type: MaterialType.SandMedSaltInnblanding,
            kategori: MaterialKategori.Sand,
        };
    }
    if (dto.materiale === "Fastsand") {
        return {
            navn: MaterialOgAktivitetsNavn.Fastsand,
            type: MaterialType.Fastsand,
            kategori: MaterialKategori.Sand,
        };
    }
    if (dto.materiale === "Tørrsalt (NaCl)") {
        return {
            navn: MaterialOgAktivitetsNavn.Tørrsalt,
            type: MaterialType.Tørrsalt,
            kategori: MaterialKategori.Salt,
        };
    }
    if (dto.materiale === "Befuktet salt (NaCl)") {
        return {
            navn: MaterialOgAktivitetsNavn.BefuktetSalt,
            type: MaterialType.BefuktetSalt,
            kategori: MaterialKategori.Salt,
        };
    }
    if (dto.materiale === "Saltslurry (NaCl)" || dto.materiale === "Saltslurry (materialKode 11)") {
        return {
            navn: MaterialOgAktivitetsNavn.BefuktetFinkornetSalt,
            type: MaterialType.BefuktetFinkornetSalt,
            kategori: MaterialKategori.Salt,
        };
    }
    if (dto.materiale === "Saltløsning (NaCl)") {
        return {
            navn: MaterialOgAktivitetsNavn.Saltløsning,
            type: MaterialType.Saltløsning,
            kategori: MaterialKategori.Salt,
        };
    }
    if (dto.materiale === "Magnesiumklorid (MgCl)") {
        return {
            navn: MaterialOgAktivitetsNavn.Magnesiumklorid,
            type: MaterialType.Magnesiumklorid,
            kategori: MaterialKategori.Salt,
        };
    }
    if (dto.materiale === "Ukjent materiale") {
        return {
            navn: MaterialOgAktivitetsNavn.UkjentMateriale,
            type: MaterialType.UkjentMateriale,
            kategori: MaterialKategori.UkjentMateriale,
        };
    }
};
