import React, { ChangeEvent, InputHTMLAttributes, useCallback, useContext } from "react";
import { CheckboxGruppeContext } from "./CheckboxGruppe";
import styled from "styled-components";
import { Tekst } from "../typografi/Tekst";

export interface CheckboxProps
    extends Omit<InputHTMLAttributes<HTMLInputElement>, "size" | "value"> {
    value: string;
    children: React.ReactNode;
    disabled?: boolean;
    checked?: boolean;
    onChange?: (e: ChangeEvent<HTMLInputElement>) => void;
    størrelse?: "XS" | "S" | "L";
    tekstStørrelse?: "S" | "M" | "L";
    automatiskFokus?: boolean;
}

export const Checkbox = ({
    value,
    children,
    disabled,
    checked,
    onChange,
    størrelse = "L",
    tekstStørrelse = "L",
    automatiskFokus = false,
}: CheckboxProps) => {
    const checkboxGroup = useContext(CheckboxGruppeContext);
    const isChecked = checkboxGroup?.value ? checkboxGroup.value.includes(value) : checked;

    const doOnChange = (e: ChangeEvent<HTMLInputElement>) => {
        onChange && onChange(e);
        checkboxGroup && checkboxGroup.toggleValue(value);
    };

    const input = useCallback((inputElement: HTMLInputElement) => {
        if (inputElement && automatiskFokus) {
            inputElement.focus();
        }
    }, []);

    return (
        <Wrapper>
            <Input
                id={value}
                type="checkbox"
                value={value}
                checked={isChecked}
                onChange={doOnChange}
                disabled={disabled}
                $størrelse={størrelse}
                ref={input}
            />
            <CheckboxLabel htmlFor={value} $størrelse={størrelse}>
                <Tekst as="span" $størrelse={tekstStørrelse}>
                    {children}
                </Tekst>
            </CheckboxLabel>
        </Wrapper>
    );
};

const Wrapper = styled.div`
    position: relative;
    width: fit-content;
`;

const Input = styled.input<{ $størrelse: string }>`
    cursor: pointer;
    height: 44px;
    width: 44px;
    opacity: 0;
    position: absolute;
    top: -9px;
    left: -12px;
    z-index: 1;

    ${(props) =>
        props.$størrelse === "S" &&
        `
        height: 26px;
        width: 26px;
        top: -2px;
        left: -3px;
        `}

    ${(props) =>
        props.$størrelse === "XS" &&
        `
        height: 13px;
        width: 13px;
        top: -2px;
        left: -3px;
        `}
`;

const CheckboxLabel = styled.label<{ $størrelse: string }>`
    align-items: center;
    cursor: pointer;
    display: flex;
    gap: 8px;

    ${Input}:disabled + & > ${Tekst} {
        color: var(--gray-regular-dark);
    }

    &:before {
        background-color: var(--white);
        border-radius: 0;
        box-shadow: inset 0 0 0 2px var(--gray-dark);
        content: "";
        flex-shrink: 0;
        height: 32px;
        width: 32px;

        ${(props) =>
            props.$størrelse === "S" &&
            `
        height: 24px;
        width: 24px;
        `}

        ${(props) =>
            props.$størrelse === "XS" &&
            `
        height: 18px;
        width: 18px;
        `}

        ${Input}:checked + & {
            background: url("/ikon/checkmark.svg") no-repeat 50%;
            background-color: var(--white);
            background-size: 60%;
        }

        ${Input}:hover + & {
            box-shadow: inset 0 0 0 3px var(--gray-dark);
        }

        ${Input}:focus + & {
            box-shadow: inset 0 0 0 3px var(--gray-dark);
            outline: 3px solid var(--orange);
            outline-offset: 2px;
        }

        ${Input}:disabled + & {
            box-shadow: inset 0 0 0 2px var(--gray-regular-dark);
        }
    }
`;
