import {
    CartesianGrid,
    Label,
    LineChart,
    ReferenceArea,
    ReferenceLine,
    ResponsiveContainer,
    Tooltip,
    XAxis,
    YAxis,
} from "recharts";
import { formatAxisTid } from "../../øktdetaljerSide/grafer/grafUtils";
import React, { ReactNode, useState } from "react";
import dayjs, { Dayjs } from "dayjs";
import styled from "styled-components";
import { breakpointNettbrett } from "../../utils/constants";
import { aktivGrafTypeAtom, reflinjeIKartAtom } from "../../øktdetaljerSide/jotai/atoms";
import { useAtomValue, useAtom } from "jotai";
import { Heading } from "../fellesKomponenter/typografi/Heading";
import {
    ØktdetaljerLoggEvent,
    loggEventMedTabellnavn,
    TabellNavn,
    FriksjonLoggEvent,
} from "../../utils/loggFrontend";
import { Tekst } from "../fellesKomponenter/typografi/Tekst";
import { RowWrapper } from "../fellesKomponenter/wrappers/wrappers";
import { KnappSekundær } from "../fellesKomponenter/knapper/KnappSekundær";
import {
    hentGraftittel,
    hentReflinjeBenevning,
    hentTicksXaxis,
    hentYAxisLabel,
} from "./fellesGrafUtils";

import { Friksjonsmåling } from "../../friksjonsdashboardSide/friksjonsmålingTyper";
import { aktivMålingTidspunktAtom } from "../../friksjonsdashboardSide/jotai/atoms";
import { CategoricalChartState } from "recharts/types/chart/types";

interface Props {
    tickArrayYaxis: number[];
    customLegend?: JSX.Element;
    customTooltip: JSX.Element;
    startTid: Dayjs;
    sluttTid: Dayjs;
    children: ReactNode;
    doseringsBenevning?: string;
    setAktivFriksjonsmåling?: (måling: Friksjonsmåling | undefined) => void;
    dataKeyXaxis: (data: any) => number;
}

export const GeneriskGraf = ({
    tickArrayYaxis,
    customLegend,
    customTooltip,
    doseringsBenevning,
    startTid,
    sluttTid,
    setAktivFriksjonsmåling,
    dataKeyXaxis,
    children,
}: Props) => {
    const aktivGrafType = useAtomValue(aktivGrafTypeAtom);
    const reflinjeIKart = useAtomValue(reflinjeIKartAtom);

    const defaultXaxisDomain = [startTid.valueOf(), sluttTid.valueOf()];
    const defaultXaxisTicks = hentTicksXaxis(startTid, sluttTid);
    const [xaxisDomain, setXaxisDomain] = useState<number[]>(defaultXaxisDomain);
    const [xaxisTicks, setXaxisTicks] = useState<number[]>(defaultXaxisTicks);
    const [seleksjonstidStart, setSeleksjonstidStart] = useState<number | undefined>(undefined);
    const [seleksjonstidSlutt, setSeleksjonstidSlutt] = useState<number | undefined>(undefined);
    const [visTilbakestillknapp, setVisTilbakestillknapp] = useState(false);

    const yAxisRefLine = useAtomValue(aktivMålingTidspunktAtom);
    const yAxisLabel = hentYAxisLabel(aktivGrafType, doseringsBenevning);
    const referanselinjeBenvevning = hentReflinjeBenevning(aktivGrafType, doseringsBenevning);
    const tittel = hentGraftittel(aktivGrafType);

    const handleSeleksjon = (e: CategoricalChartState) => {
        if (
            seleksjonstidStart === seleksjonstidSlutt ||
            seleksjonstidSlutt === undefined ||
            seleksjonstidStart === undefined
        ) {
            if (setAktivFriksjonsmåling) {
                e.activePayload && setAktivFriksjonsmåling(e.activePayload[0].payload);
                loggEventMedTabellnavn(TabellNavn.grafvalgFriksjon, FriksjonLoggEvent.klikkIGraf);
            }
            setSeleksjonstidStart(undefined);
            setSeleksjonstidSlutt(undefined);
            return;
        }

        const domain = [seleksjonstidStart, seleksjonstidSlutt].sort();
        setXaxisDomain(domain);
        setXaxisTicks(hentTicksXaxis(dayjs(domain[0]), dayjs(domain[1])));
        setSeleksjonstidSlutt(undefined);
        setSeleksjonstidStart(undefined);
        setVisTilbakestillknapp(true);
        if (setAktivFriksjonsmåling) {
            loggEventMedTabellnavn(TabellNavn.friksjonsdashboard, FriksjonLoggEvent.zoom);
        } else {
            loggEventMedTabellnavn(TabellNavn.øktdetaljer, ØktdetaljerLoggEvent.zoom);
        }
    };

    const handleReset = () => {
        setXaxisDomain(defaultXaxisDomain);
        setXaxisTicks(defaultXaxisTicks);
        setVisTilbakestillknapp(false);
    };

    return (
        <div>
            <GrafContainer>
                <CustomHeading $stylelevel={"4"} as={"h4"}>
                    {tittel}
                </CustomHeading>
                {customLegend}
                <ResponsiveContainer height={360} minWidth={550}>
                    <LineChart
                        margin={{ top: 50, right: 40, left: 20, bottom: 30 }}
                        style={{ userSelect: "none" }}
                        onMouseDown={(e) =>
                            e !== null && setSeleksjonstidStart(Number(e.activeLabel))
                        }
                        onMouseMove={(e) =>
                            seleksjonstidStart &&
                            e !== null &&
                            setSeleksjonstidSlutt(Number(e.activeLabel))
                        }
                        onMouseUp={(e) => e !== null && handleSeleksjon(e)}
                    >
                        <CartesianGrid strokeDasharray="3 3" />
                        {children}
                        <ReferenceLine x={yAxisRefLine} stroke="#20770c" strokeWidth={3} />
                        <XAxis
                            allowDataOverflow
                            dataKey={dataKeyXaxis}
                            type="number"
                            tickFormatter={formatAxisTid}
                            allowDuplicatedCategory={false}
                            ticks={xaxisTicks}
                            domain={xaxisDomain}
                            user-select={"none"}
                        >
                            <Label value="Klokkeslett" offset={-15} position="insideBottom" />
                        </XAxis>
                        <YAxis domain={[0, Math.max(...tickArrayYaxis)]} ticks={tickArrayYaxis}>
                            <Label
                                value={yAxisLabel}
                                offset={15}
                                position="insideBottomLeft"
                                angle={-90}
                            />
                        </YAxis>
                        <Tooltip content={customTooltip} />

                        {seleksjonstidStart && seleksjonstidSlutt ? (
                            <ReferenceArea
                                x1={seleksjonstidStart}
                                x2={seleksjonstidSlutt}
                                strokeOpacity={0.3}
                            />
                        ) : null}
                        {reflinjeIKart && (
                            <ReferenceLine
                                y={reflinjeIKart}
                                label={{
                                    value: reflinjeIKart + referanselinjeBenvevning,
                                    position: "top",
                                    fontSize: 16,
                                    offset: 10,
                                    fontWeight: "bold",
                                    fill: "red",
                                }}
                                stroke="red"
                                strokeDasharray="3 3"
                                strokeWidth={3}
                                isFront={true}
                            />
                        )}
                    </LineChart>
                </ResponsiveContainer>
            </GrafContainer>
            <TilbakestillContainer>
                <Innhold>
                    <Tekst $nomargin={true} $størrelse={"M"}>
                        Zoom i grafen ved å klikke å dra ønsket intervall
                    </Tekst>
                    {visTilbakestillknapp && (
                        <RowWrapper>
                            <KnappSekundær onClick={handleReset} size={"Small"}>
                                Tilbakestill graf
                            </KnappSekundær>
                        </RowWrapper>
                    )}
                </Innhold>
            </TilbakestillContainer>
        </div>
    );
};

const GrafContainer = styled.div`
    @media (max-width: ${breakpointNettbrett}) {
        overflow-x: scroll;
        overflow-y: hidden;
    }
`;

const CustomHeading = styled(Heading)`
    display: flex;
    justify-content: center;
    margin-bottom: 10px;
`;

const TilbakestillContainer = styled.div`
    display: flex;
    flex-direction: column;
    width: 100%;
    background-color: var(--gray-light);
    height: fit-content;
`;

const Innhold = styled.div`
    display: flex;
    flex-direction: column;
    padding: 30px 30px 0 30px;
    gap: 20px;
`;
