import { SisteKjøretøyDto } from "../../api/useHentSisteKjøretøy";
import React, { useEffect, useState } from "react";
import { MapContainer, TileLayer } from "react-leaflet";
import styles from "./Kartvisning.module.css";
import { crsUtm33N } from "../utils/crs";
import { VeglenkePopupInfo } from "../popup/veglenke/VeglenkePopup";
import { KameraCluster } from "../marker/KameraCluster";
import { ChangeView } from "./ChangeView";
import { KjøretøyCluster } from "../marker/KjøretøyCluster";
import { ZoomKnapp } from "./ZoomKnapp";
import { RodeDto } from "../../api/useHentRoder";
import { TrafikkmeldingerDto, useHentTrafikkmeldinger } from "../../api/useHentTrafikkmeldinger";
import {
    informasjonsmeldinger,
    informasjonsmeldingMedTimeoutAtom,
} from "../jotai/InformasjonsmeldingAtom";
import { TrafikkmeldingClusterWrapper } from "./TrafikkmeldingClusterWrapper";
import { LatLngBoundsExpression } from "leaflet";
import { VisMenyKnapp } from "../../components/fellesKomponenter/uttrekksmeny/VisMenyKnapp";
import { KartParameterVelger } from "../kartParameterVelger/KartParameterVelger";
import { KartLagIkon } from "../../components/Ikon/KartLagIkon";
import { KartKnapp } from "../../components/fellesKomponenter/knapper/KartKnapp";
import { KartlagValg } from "./KartlagValg";
import { Informasjonsmelding } from "../feil/Informasjonsmelding";
import { IntervallLegend } from "./IntervallLegend/IntervallLegend";
import { AnimatePresence } from "framer-motion";
import { useUntabableClusters } from "../../hooks/useUntabableClusters";
import { HoldClusterUtabbbar } from "./HoldClusterUtabbbar";
import { createGlobalStyle } from "styled-components";
import { useWindowSize } from "../../hooks/useWindowSize";
import { pilMenyWidth } from "../../utils/constants";
import { useAtom, useAtomValue, useSetAtom } from "jotai";
import {
    visBrøytebilerAtom,
    visLegendAtom,
    visDarkModeAtom,
    visKameraerAtom,
    visTrafikkmeldingerAtom,
    menyÅpenAtom,
} from "../jotai/karvalgAtoms";
import { Kamera } from "../../api/useHentVærstasjoner";
import { tidspunktAtom } from "../../api/context/kontraktAtoms";

export enum InformasjonsPopupType {
    VEGLENKE,
    KJØRETØY,
    KAMERA,
    RODE,
    TRAFIKKMELDING,
    UKJENT,
}

export type PopupType =
    | VeglenkePopupInfo
    | Kamera
    | SisteKjøretøyDto
    | RodeDto
    | TrafikkmeldingerDto;

export interface GenrellKartvisningProps {
    kjøretøydata: SisteKjøretøyDto[];
    kameradata: Kamera[];
    kartBounds: LatLngBoundsExpression;
    children: React.ReactNode;
}

export const KartvisningGenerell = ({
    kjøretøydata,
    kameradata,
    kartBounds,
    children,
}: GenrellKartvisningProps) => {
    const tidspunkt = useAtomValue(tidspunktAtom);
    const setInformasjonsmelding = useSetAtom(informasjonsmeldingMedTimeoutAtom);
    const [menyÅpen, setMenyÅpen] = useAtom(menyÅpenAtom);
    const trafikkmeldinger = useHentTrafikkmeldinger(tidspunkt);
    useUntabableClusters();

    const visKameraer = useAtomValue(visKameraerAtom);
    const [visTrafikkmeldinger, setVisTrafikkmeldinger] = useAtom(visTrafikkmeldingerAtom);
    const visBrøytebiler = useAtomValue(visBrøytebilerAtom);
    const visLegend = useAtomValue(visLegendAtom);
    const visDarkmode = useAtomValue(visDarkModeAtom);
    const [visKartlagValg, setVisKartlagValg] = useState(false);

    const { windowWidth } = useWindowSize();
    const menywidth = windowWidth < 330 + pilMenyWidth ? windowWidth - pilMenyWidth : 330;

    useEffect(() => {
        window.addEventListener("hashchange", åpneMenyPåFokusPåHovedinnhold);
        return () => window.removeEventListener("hashchange", åpneMenyPåFokusPåHovedinnhold);
    }, []);

    const handleTrafikkmeldingToggle = () => {
        if (!trafikkmeldinger) {
            setInformasjonsmelding(informasjonsmeldinger.lasterStengteVeier);
        } else if (!trafikkmeldinger?.suksess || trafikkmeldinger.data.length === 0) {
            setInformasjonsmelding(informasjonsmeldinger.ingenDataForStengteVeier);
        } else {
            setVisTrafikkmeldinger(!visTrafikkmeldinger);
        }
    };
    const åpneMenyPåFokusPåHovedinnhold = () => {
        if (location.hash === "#kartparametervelger") {
            setMenyÅpen(true);
        }
    };

    return (
        <MapContainer
            className={styles.kartvisning}
            center={[64.6, 9.5022]}
            scrollWheelZoom
            crs={crsUtm33N}
            touchZoom={true}
            minZoom={4}
            maxZoom={16}
            zoomControl={false}
            keyboard={false}
            attributionControl={false}
        >
            <TileLayer
                attribution="© NVDB, Geovekst, kommunene og Open Street Map contributors (utenfor Norge)"
                url="https://services.geodataonline.no/arcgis/rest/services/Trafikkportalen/GeocacheTrafikkJPG/MapServer/tile/{z}/{y}/{x}"
                subdomains="123456789"
                className={visDarkmode ? "darkmode-kart" : ""}
                key={visDarkmode ? "darkmodeKart" : "lightmodeKart"}
            />

            {children}

            {kjøretøydata.length > 0 && visBrøytebiler && (
                <KjøretøyCluster kjøretøyListe={kjøretøydata} />
            )}

            {kameradata.length > 0 && visKameraer && <KameraCluster kameradata={kameradata} />}

            {trafikkmeldinger?.suksess && visTrafikkmeldinger && (
                <TrafikkmeldingClusterWrapper trafikkmeldinger={trafikkmeldinger.data} />
            )}
            <AnimatePresence>{menyÅpen && <KartParameterVelger />}</AnimatePresence>

            {!menyÅpen && <VisMenyKnapp onClick={() => setMenyÅpen(true)} />}

            <MenyVariables width={menywidth} />

            <KartKnapp
                id={"karlagvalg"}
                $right={20}
                $bottom={126}
                onClick={() => setVisKartlagValg(!visKartlagValg)}
            >
                <KartLagIkon />
            </KartKnapp>

            <AnimatePresence>
                {visKartlagValg && (
                    <KartlagValg
                        åpen={visKartlagValg}
                        handleTrafikkmeldingToggle={handleTrafikkmeldingToggle}
                        trafikkmeldinger={trafikkmeldinger}
                        onClose={() => setVisKartlagValg(false)}
                    />
                )}
            </AnimatePresence>
            <ZoomKnapp size="large" />
            {visLegend && <IntervallLegend />}
            <Informasjonsmelding />
            <ChangeView kartBounds={kartBounds} />
            <HoldClusterUtabbbar />
        </MapContainer>
    );
};

const MenyVariables = createGlobalStyle<{ width: number }>`
  &:root {
    --meny-width: ${({ width }) => width}px;
  }
`;
