import { SisteKjøretøyDto } from "../../api/useHentSisteKjøretøy";
import React, { useEffect, useMemo } from "react";
import { VeglenkePopup, VeglenkePopupInfo } from "../popup/veglenke/VeglenkePopup";
import { Feature } from "geojson";
import { LeafletEvent } from "leaflet";
import { Produksjonslenke } from "../types/produksjonslenke";
import { RodeDto } from "../../api/useHentRoder";
import { getPopupType, mapTilRodePopupInfo, mapTilVeglenkePopupInfo } from "../popup/popupUtils";
import { finnKartBoundsForProduksjonskart } from "./kartBoundsUtils";
import { InformasjonsPopupType, KartvisningGenerell } from "./KartvisningGenerell";
import { useTomProduksjonsvarsel } from "../../hooks/useTomProduksjonsvarsel";
import { RodePopup, RodePopupMedProduksjonInfo } from "../popup/roder/RodePopup";
import { useFiltrerteLenker } from "./UseFiltrerteLenker";
import { AnimatePresence } from "framer-motion";
import { GeoJsonRoder } from "./geoJson/GeoJsonRoder";
import { GeoJsonVeglenker } from "./geoJson/GeoJsonVeglenker";
import { hentRodeIdFraVeglenke } from "../utils/kartUtils";
import { useAtom, useAtomValue } from "jotai";
import {
    aktivRodeFeatureAtom,
    aktivVeglenkeFeatureAtom,
    popupInfoAtom,
    visProduksjonAtom,
    visRoderAtom,
} from "../jotai/karvalgAtoms";
import { Kamera } from "../../api/useHentVærstasjoner";
import { loggEventMedTabellnavn, PopupLoggEvent, TabellNavn } from "../../utils/loggFrontend";

export interface ProduksjonskartVisningProps {
    produksjonslenker: Produksjonslenke[];
    kjøretøydata: SisteKjøretøyDto[];
    roder: RodeDto[];
    kameradata: Kamera[];
}

export const ProduksjonskartVisning = ({
    produksjonslenker,
    kjøretøydata,
    kameradata,
    roder,
}: ProduksjonskartVisningProps) => {
    const [aktivRodeFeature, settAktivRodeFeature] = useAtom(aktivRodeFeatureAtom);
    const [aktivVeglenkeFeature, settAktivVeglenkeFeature] = useAtom(aktivVeglenkeFeatureAtom);

    const [popupInfo, settPopupInfo] = useAtom(popupInfoAtom);
    const visRoder = useAtomValue(visRoderAtom);
    const visProduksjon = useAtomValue(visProduksjonAtom);
    const kartbounds = useMemo(
        () => finnKartBoundsForProduksjonskart(roder, produksjonslenker),
        [roder, produksjonslenker],
    );

    useTomProduksjonsvarsel();
    const filtrerteProduksjonslenker = useFiltrerteLenker(produksjonslenker);

    useEffect(() => {
        if (popupInfo && getPopupType(popupInfo) !== InformasjonsPopupType.VEGLENKE) {
            settAktivVeglenkeFeature(undefined);
        }
    }, [popupInfo]);

    useEffect(() => {
        const rodeIdPåVeglenke = hentRodeIdFraVeglenke(
            aktivVeglenkeFeature?.properties?.veglenkesekvensId,
            aktivVeglenkeFeature?.properties?.veglenkeNummer,
            roder,
        );
        if (
            aktivRodeFeature?.id &&
            aktivVeglenkeFeature &&
            !rodeIdPåVeglenke.includes(aktivRodeFeature.id.toString())
        ) {
            settAktivRodeFeature(undefined);
        }
    }, [aktivVeglenkeFeature]);

    const håndterVeglenkeKlikket = (event: LeafletEvent, feature: Feature) => {
        settPopupInfo(mapTilVeglenkePopupInfo(event, feature));
        settAktivVeglenkeFeature(feature);
        loggEventMedTabellnavn(TabellNavn.popupTrykk, PopupLoggEvent.veglenke);
    };

    const håndterRodeKlikket = (event: LeafletEvent, feature: Feature) => {
        settAktivRodeFeature(feature);
        settPopupInfo(mapTilRodePopupInfo(event, feature));
        loggEventMedTabellnavn(TabellNavn.popupTrykk, PopupLoggEvent.rodelenke);
    };

    return (
        <KartvisningGenerell
            kjøretøydata={kjøretøydata}
            kameradata={kameradata}
            kartBounds={kartbounds}
        >
            <>
                {visRoder && (
                    <GeoJsonRoder
                        roder={roder}
                        aktivRodeFeature={aktivRodeFeature}
                        håndterRodeKlikket={håndterRodeKlikket}
                    />
                )}

                {visProduksjon && (
                    <GeoJsonVeglenker
                        filtrerteProduksjonslenker={filtrerteProduksjonslenker}
                        håndterVeglenkeKlikket={håndterVeglenkeKlikket}
                        aktivVeglenkeFeature={aktivVeglenkeFeature}
                    />
                )}
                <AnimatePresence>
                    {getPopupType(popupInfo) === InformasjonsPopupType.RODE && (
                        <RodePopup rodePopupInfo={popupInfo as RodePopupMedProduksjonInfo} />
                    )}
                </AnimatePresence>
                <AnimatePresence>
                    {getPopupType(popupInfo) === InformasjonsPopupType.VEGLENKE && (
                        <VeglenkePopup
                            veglenkePopupInfo={popupInfo as VeglenkePopupInfo}
                            produksjonslenker={produksjonslenker}
                        />
                    )}
                </AnimatePresence>
            </>
        </KartvisningGenerell>
    );
};
