import { Produksjonslenke } from "../types/produksjonslenke";
import { useState } from "react";
import { ApiRespons } from "../../api/api";
import {
    KartparametereLoggEvent,
    loggEventMedTabellnavn,
    loggFrontend,
    LoggNivå,
    TabellNavn,
} from "../../utils/loggFrontend";
import styled from "styled-components";
import { downloadExcel, EksporterTilExcel } from "../../utils/EksporterTilExcel";
import { Spinner } from "../../components/fellesKomponenter/spinner/Spinner";
import { Tekst } from "../../components/fellesKomponenter/typografi/Tekst";
import { Tekstpil } from "../../components/Ikon/Tekstpil";
import { RodeDto } from "../../api/useHentRoder";
import {
    informasjonsmeldinger,
    informasjonsmeldingMedTimeoutAtom,
} from "../jotai/InformasjonsmeldingAtom";
import { KnappUtenStyling } from "../../components/fellesKomponenter/knapper/KnappUtenStyling";
import { Produksjonsdetaljer } from "../types/produksjonsdetaljer";
import { filtrerProduksjonsdetaljer } from "../filtre/filtrerProduksjon/filtrerProduksjon";
import { useAtomValue, useSetAtom } from "jotai";
import { tidspunktAtom } from "../../api/context/kontraktAtoms";
import { tidsfilterAtom } from "../jotai/TidsfilterAtom";
import { hastighetsfilterAtom } from "../jotai/HastighetsfilterAtom";
import { produksjonsfilterAtom } from "../../api/context/ProduksjonsfilterAtoms";
import { roderAtom } from "../../api/context/apiAtoms";
import { hentProduksjonsdetaljer } from "../../api/useHentProduksjonsdetaljer";

interface EksporterAllDataProps {
    produksjonslenker: ApiRespons<Produksjonslenke[]> | undefined;
}
export const EksporterAllData = ({ produksjonslenker }: EksporterAllDataProps) => {
    const tidspunkt = useAtomValue(tidspunktAtom);
    const produksjonsfilter = useAtomValue(produksjonsfilterAtom);
    const hastighetsfilter = useAtomValue(hastighetsfilterAtom);
    const tidsfilter = useAtomValue(tidsfilterAtom);
    const [loading, setLoading] = useState<boolean>(false);
    const roder = useAtomValue(roderAtom);
    const setInformasjonsmelding = useSetAtom(informasjonsmeldingMedTimeoutAtom);

    const valgteFiltere = produksjonsfilter.filter((f) => f.valgt).map((f) => f.navn);
    const hentMålepunkter = () => {
        const punkter = produksjonslenker?.data.flatMap(
            (produksjonslenke: Produksjonslenke) => produksjonslenke.målepunkter,
        );

        if (punkter.length === 0) {
            const loggInfo = `Ingen produksjonsdetaljer for kontrakt på valgt tidspunkt,\n tidspunkt: ${tidspunkt.toISOString()},\n valgteFiltere:${valgteFiltere.toString()}`;
            loggFrontend({
                loggNivå: LoggNivå.Warn,
                loggMelding: loggInfo,
            });
        }

        return punkter;
    };

    const filtrerteProduksjonsdetaljer = (produksjonsdetaljer: Produksjonsdetaljer[]) =>
        filtrerProduksjonsdetaljer(
            produksjonsfilter,
            hastighetsfilter,
            tidsfilter,
            produksjonsdetaljer,
        );
    const hentData = async (roder: RodeDto[] | undefined) => {
        loggEventMedTabellnavn(TabellNavn.kartparametere, KartparametereLoggEvent.lastned);
        setLoading(true);
        const målepunkter = hentMålepunkter();
        const produksjonsdetaljer = await hentProduksjonsdetaljer(målepunkter, tidspunkt);
        if (produksjonsdetaljer?.suksess) {
            if (produksjonsdetaljer.data.length === 0) {
                setInformasjonsmelding(informasjonsmeldinger.ingenDataIKartet);
            } else
                await downloadExcel(
                    produksjonsdetaljer.data,
                    filtrerteProduksjonsdetaljer(produksjonsdetaljer.data),
                    [],
                    valgteFiltere,
                    roder,
                );
        }
        setLoading(false);
    };

    return (
        <VisSkjulKomponent>
            <Innhold>
                <Tekst $størrelse={"S"} $nomargin={true}>
                    Du kan laste ned det du ser i kartet som regneark til Excel. NB! Det kan ta opp
                    mot 2 minutter.
                </Tekst>
                {loading ? (
                    <HenterData />
                ) : (
                    produksjonslenker?.suksess && (
                        <EksporterTilExcel
                            tekst={"Last ned"}
                            onClick={() => hentData(roder?.data)}
                        />
                    )
                )}
            </Innhold>
        </VisSkjulKomponent>
    );
};

interface VisSkjulProps {
    children: React.ReactNode;
}
const VisSkjulKomponent = ({ children }: VisSkjulProps) => {
    const [åpen, settÅpen] = useState<boolean>(false);
    return (
        <Container>
            <KnappUtenStyling onClick={() => settÅpen(!åpen)}>
                <Tekst $nomargin={true} $størrelse={"M"} $bold={true}>
                    Last ned data til Excel?
                </Tekst>
                <Tekstpil størrelse={14} $rotert={åpen} />
            </KnappUtenStyling>
            {åpen && children}
        </Container>
    );
};

const HenterData = () => (
    <HenterDataWrapper>
        <Spinner inline={true} storrelse={"xliten"} />
        <Tekst $størrelse={"S"} $nomargin={true}>
            Laster ned...
        </Tekst>
    </HenterDataWrapper>
);

const Innhold = styled.div`
    display: flex;
    flex-direction: column;
    gap: 8px;
`;

const Container = styled.div`
    margin-top: 0.5rem;
    display: flex;
    flex-direction: column;
`;

const HenterDataWrapper = styled.div`
    display: flex;
    flex-direction: row;
    justify-content: flex-start;
    align-items: center;
    gap: 4px;
    padding: 0;
`;
