import {
    finnInitAktivKontrakt,
    finnInitTidspunkt,
    hentKartUrlParam,
    settKartUrlParam,
} from "./kartUrlUtils";
import { fraEnkelTimestampStr, tidsformat } from "../../utils/tid";
import dayjs from "dayjs";
import { sideInfo } from "../../sider";

describe("RodeKart url utils", () => {
    const gyldigKontraktStr = "10";
    const gyldigTimestampStr = "2000-05-17+17%3A30";

    describe("Håndtering av URL-parametre", () => {
        describe("Verdier settes og formateres korrekt som urlparametre ", () => {
            it("setter ikke parametre ved annen path enn produksjonskart", () => {
                settKartUrlParam(sideInfo.webkamera.href, {
                    elrappKontraktId: 10,
                    timestamp: fraEnkelTimestampStr("2000-05-17 17:30"),
                });
                expect(global.window.location.search).toEqual("");
            });
            it("setter parametre ved produksjonskart som path", () => {
                settKartUrlParam(sideInfo.produksjonskart.href, {
                    elrappKontraktId: 10,
                    timestamp: fraEnkelTimestampStr("2000-05-17 17:30"),
                });
                expect(global.window.location.search).toEqual(
                    `?elrappkontrakt=10&tidspunkt=2000-05-17+17%3A30`,
                );
            });
        });

        describe("Verdier hentes og parses fra urlparametre", () => {
            it("henter og parser gyldige urlparametre", () => {
                history.replaceState(
                    "",
                    "",
                    `?elrappkontrakt=${gyldigKontraktStr}&tidspunkt=${gyldigTimestampStr}`,
                );
                const parsedeParametre = hentKartUrlParam();
                expect(parsedeParametre?.timestamp).toStrictEqual(
                    fraEnkelTimestampStr("2000-05-17 17:30"),
                );
                expect(parsedeParametre?.elrappKontraktId).toStrictEqual(10);
            });
            it("ignorerer ukjente parametre", () => {
                history.replaceState(
                    "",
                    "",
                    `?elrappkontrakt=${gyldigKontraktStr}&tidspunkt=${gyldigTimestampStr}&ukjentparameter=idk`,
                );
                const parsedeParametre = hentKartUrlParam();
                expect(parsedeParametre?.timestamp).toStrictEqual(
                    fraEnkelTimestampStr("2000-05-17 17:30"),
                );
                expect(parsedeParametre?.elrappKontraktId).toStrictEqual(10);
            });
            it("gir undefined ved manglende kontraktid", () => {
                history.replaceState("", "", `?tidspunkt=${gyldigTimestampStr}`);
                const ingentingHentet = hentKartUrlParam();
                expect(ingentingHentet).toBe(undefined);
            });
            it("gir undefined ved manglende tidspunkt", () => {
                history.replaceState(
                    "",
                    "",
                    `?elrappkontrakt=${gyldigKontraktStr}&paramsomikkegjenkjennes=${gyldigTimestampStr}`,
                );
                const ingentingHentet = hentKartUrlParam();
                expect(ingentingHentet).toBe(undefined);
            });
            it("gir undefined ved ugyldig kontraktid", () => {
                history.replaceState(
                    "",
                    "",
                    `?elrappkontrakt=ugyldigTekst&tidspunkt=${gyldigTimestampStr}`,
                );
                const ingentingHentet = hentKartUrlParam();
                expect(ingentingHentet).toBe(undefined);
            });
            it("gir undefined ved ugyldig tidspunkt", () => {
                history.replaceState(
                    "",
                    "",
                    `?elrappkontrakt=${gyldigKontraktStr}&tidspunkt=ugyldigTidspunkt`,
                );
                const ingentingHentet = hentKartUrlParam();
                expect(ingentingHentet).toBe(undefined);
            });
        });
    });

    describe("Initialisering av state", () => {
        describe("Initialisering av tidspunktstate", () => {
            it("gir gyldig URL-dato i fortid som initielt tidspunkt", () => {
                history.replaceState(
                    "",
                    "",
                    `?elrappkontrakt=${gyldigKontraktStr}&tidspunkt=${gyldigTimestampStr}`,
                );
                const tidspunkt = finnInitTidspunkt();
                const forventetTidspunkt = fraEnkelTimestampStr("2000-05-17 17:30");
                expect(tidspunkt).toEqual(forventetTidspunkt);
            });
            it("gir default (nåtid) ved gyldig URL-dato i fremtid", () => {
                history.replaceState(
                    "",
                    "",
                    `?elrappkontrakt=${gyldigKontraktStr}&tidspunkt="2050-05-17+17%3A30"`,
                );
                expect(finnInitTidspunkt().format(tidsformat.ENKEL_TIMESTAMP)).toEqual(
                    dayjs().format(tidsformat.ENKEL_TIMESTAMP),
                );
            });
            it("gir default (nåtid) ved ugyldig URL-dato", () => {
                history.replaceState(
                    "",
                    "",
                    `?elrappkontrakt=${gyldigKontraktStr}&tidspunkt="abc"`,
                );
                expect(finnInitTidspunkt().format(tidsformat.ENKEL_TIMESTAMP)).toEqual(
                    dayjs().format(tidsformat.ENKEL_TIMESTAMP),
                );
            });
            it("gir default (nåtid) ved manglende URL-dato", () => {
                history.replaceState("", "", `?elrappkontrakt=${gyldigKontraktStr}"`);
                expect(finnInitTidspunkt().format(tidsformat.ENKEL_TIMESTAMP)).toEqual(
                    dayjs().format(tidsformat.ENKEL_TIMESTAMP),
                );
            });
        });
        describe("Initialisering av kontraktstate", () => {
            const mockKontrakter = [
                {
                    navn: "Kontrakt B",
                    navnNvdb: "1234 Kontrakt B",
                    elrappId: 123,
                    utgått: false,
                },
                {
                    navn: "Kontrakt A",
                    navnNvdb: "1223 Kontrakt B",
                    elrappId: Number(gyldigKontraktStr),
                    utgått: false,
                },
            ];
            it("gir gyldig URL-kontraktid som initiell kontrakt", () => {
                history.replaceState(
                    "",
                    "",
                    `?elrappkontrakt=${gyldigKontraktStr}&tidspunkt=${gyldigTimestampStr}`,
                );
                const initAktivKontrakt = finnInitAktivKontrakt(mockKontrakter, mockKontrakter[0]);
                expect(initAktivKontrakt?.elrappId).toEqual(mockKontrakter[1].elrappId);
                expect(initAktivKontrakt?.navn).toEqual(mockKontrakter[1].navn);
            });
            it("gir default (første brukers kontrakt) ved URL-kontraktid hvor bruker ikke har tilgang", () => {
                history.replaceState(
                    "",
                    "",
                    `?elrappkontrakt=9999&tidspunkt=${gyldigTimestampStr}`,
                );
                const initAktivKontrakt = finnInitAktivKontrakt(mockKontrakter, mockKontrakter[0]);
                expect(initAktivKontrakt?.elrappId).toEqual(mockKontrakter[0].elrappId);
                expect(initAktivKontrakt?.navn).toEqual(mockKontrakter[0].navn);
            });
            it("gir default (første brukers kontrakt) ved ugyldig URL-kontraktid", () => {
                history.replaceState("", "", `?elrappkontrakt=abc&tidspunkt=${gyldigTimestampStr}`);
                const initAktivKontrakt = finnInitAktivKontrakt(mockKontrakter, mockKontrakter[0]);
                expect(initAktivKontrakt?.elrappId).toEqual(mockKontrakter[0].elrappId);
                expect(initAktivKontrakt?.navn).toEqual(mockKontrakter[0].navn);
            });
            it("gir default (første brukers kontrakt) ved manglende URL-kontraktid", () => {
                history.replaceState("", "", `?tidspunkt=${gyldigTimestampStr}`);
                const initAktivKontrakt = finnInitAktivKontrakt(mockKontrakter, mockKontrakter[0]);
                expect(initAktivKontrakt?.elrappId).toEqual(mockKontrakter[0].elrappId);
                expect(initAktivKontrakt?.navn).toEqual(mockKontrakter[0].navn);
            });
        });
    });
});
