import dayjs from "dayjs";
import customParseFormat from "dayjs/plugin/customParseFormat";
import nb from "dayjs/locale/nb";

export type Dayjs = dayjs.Dayjs;
dayjs.extend(customParseFormat);
dayjs.locale(nb);

export enum tidsformat {
    ISO_DATO = "YYYY-MM-DD",
    ENKELT_KLOKKESLETT = "HH:mm",
    ENKEL_TIMESTAMP = "YYYY-MM-DD HH:mm",
    NORSK_TIMESTAMP = "DD.MM.YYYY HH:mm",
    DATO_OG_TID = "DD.MM HH:mm",
    FRIKSJONS_TIMESTAMP = "DD/MM/YYYY HH:mm",
    FRIKSJONS_TIMESTAMP_ENKELT_TALL = "D/M/YYYY HH:mm",
}

export const timestampStringTilEnkeltKlokkeslett = (timestamp: string): string =>
    tilEnkelKlokkeslettString(dayjs(timestamp));

export const epochTilDatoOgTid = (epoch: number): string =>
    dayjs.unix(epoch).format(tidsformat.NORSK_TIMESTAMP);

export const epochTilEnkelTimestamp = (epoch: number): string =>
    dayjs.unix(epoch).format(tidsformat.ENKEL_TIMESTAMP);

export const timestampStringTilDatoOgTid = (timestamp: string): string =>
    tilTidOgDatoString(dayjs(timestamp));
export const timestampStringTilNorsk = (timestamp: string): string =>
    tilNorskTimestamp(dayjs(timestamp));
export const tilNorskTimestamp = (timestamp: Dayjs): string =>
    timestamp.format(tidsformat.NORSK_TIMESTAMP);
export const fraEnkelKlokkeslettString = (klokkeslett: string): Dayjs =>
    dayjs(klokkeslett, tidsformat.ENKELT_KLOKKESLETT);
export const tilEnkelKlokkeslettString = (timestamp: Dayjs): string =>
    timestamp.format(tidsformat.ENKELT_KLOKKESLETT);
export const fraIsoDatoString = (dato: string): Dayjs => dayjs(dato);
export const tilIsoDatoString = (dato: Dayjs): string => dato.format(tidsformat.ISO_DATO);
export const isoDatoErIDag = (dato: string): boolean => {
    return fraIsoDatoString(dato).isSame(tilIsoDatoString(dayjs()));
};
export const fraEnkelTimestampStr = (timestamp: string): Dayjs =>
    dayjs(timestamp, tidsformat.ENKEL_TIMESTAMP);

export const fraFriksjonsTimestampStr = (timestamp: string): Dayjs => {
    const datoSplit = timestamp.split("/");
    if (datoSplit[0].trim().length === 1 || datoSplit[1].trim().length === 1) {
        return dayjs(timestamp, tidsformat.FRIKSJONS_TIMESTAMP_ENKELT_TALL);
    }
    return dayjs(timestamp, tidsformat.FRIKSJONS_TIMESTAMP);
};

export const fraOracleTimestampTilNorskString = (timestamp: string): string => {
    return tilNorskTimestamp(dayjs(timestamp, tidsformat.FRIKSJONS_TIMESTAMP));
};
export const tilEnkelTimestampStr = (timestamp: Dayjs): string =>
    timestamp.format(tidsformat.ENKEL_TIMESTAMP);
export const tilTidOgDatoString = (timestamp: Dayjs): string =>
    timestamp.format(tidsformat.DATO_OG_TID);

export const erMerEnnEtDøgnSiden = (tidspunkt: Dayjs): boolean => {
    const nå = dayjs();
    return nå.diff(tidspunkt, "minute") > 24 * 60;
};

export const erIDag = (tidspunkt: Dayjs): boolean => dayjs().isSame(tidspunkt, "day");

export const erSammeTime = (tidspunkt: Dayjs): boolean => dayjs().isSame(tidspunkt, "hour");
export const erSammeMinutt = (tidspunkt: Dayjs): boolean => dayjs().isSame(tidspunkt, "minute");
export const erSammeDag = (tidspunkt1: Dayjs, tidspunkt2: Dayjs): boolean =>
    dayjs(tidspunkt1).isSame(tidspunkt2, "day");

export const minutterTilTimerOgMinutter = (minutterTotalt?: number): string | undefined => {
    if (minutterTotalt === undefined) return undefined;

    const timer = Math.floor(minutterTotalt / 60);
    const minutter = minutterTotalt % 60;

    const timerString = timer === 0 ? "" : timer === 1 ? "1 time" : `${timer} timer`;
    const minutterString =
        minutter === 0 ? "" : minutter === 1 ? "1 minutt" : `${minutter} minutter`;

    return timerString + (timer && minutter ? " " : "") + minutterString;
};

export const varighetIDagTimeMinutt = (fra: string, til: string) => {
    const start = dayjs(fra);
    const slutt = dayjs(til);
    let antallMinutter = slutt.diff(start, "minutes");
    const antallDager = Math.floor(antallMinutter / (24 * 60));
    antallMinutter = antallMinutter - antallDager * 24 * 60;
    const antallTimer = Math.floor(antallMinutter / 60);
    antallMinutter = antallMinutter - antallTimer * 60;

    const dager =
        antallDager > 0 ? antallDager.toString() + (antallDager === 1 ? " dag, " : " dager, ") : "";
    const timer =
        antallTimer > 0 ? antallTimer.toString() + (antallTimer === 1 ? " time " : " timer ") : "";
    const minutter =
        antallMinutter > 0
            ? antallMinutter.toString() + (antallMinutter === 1 ? " minutt" : " minutter")
            : "";
    const og = (dager !== "" || timer !== "") && minutter !== "" ? "og " : "";
    return dager + timer + og + minutter;
};

export const hentUkedag = (dato: string) => {
    const ukedagnr = dayjs(dato).day();
    const dager = ["Søndag", "Mandag", "Tirsdag", "Onsdag", "Torsdag", "Fredag", "Lørdag"];
    return dager[ukedagnr];
};

export const hentUkedagOgDato = (dato: string) => {
    return hentUkedag(dato) + " " + tilNorskTimestamp(dayjs(dato));
};
