import { WEBKAMERA_URL } from "../utils/baseUrl";
import React, { useState } from "react";
import styled from "styled-components";
import { WebkameraModal } from "./WebkameraModal";
import { EksternLenke } from "../components/fellesKomponenter/lenker/EksternLenke";
import { Heading } from "../components/fellesKomponenter/typografi/Heading";
import { KnappUtenStyling } from "../components/fellesKomponenter/knapper/KnappUtenStyling";
import { useWindowSize } from "../hooks/useWindowSize";
import { KameraStørrelse } from "./types";
import {
    loggEventMedTabellnavnOgVerdi,
    TabellNavn,
    WebkameraLoggEvent,
} from "../utils/loggFrontend";

interface KameraPanelProps {
    id: number;
    stedsnavn: string;
    veg: string;
    info: string;
    vaervarsel: string;
    håndterFavorittKlikk: (id: string, navn: string, erFavoritt: boolean) => void;
    favoritter?: string[];
    størrelse: KameraStørrelse;
}

export const KameraPanel = ({
    id,
    stedsnavn,
    veg,
    info,
    vaervarsel,
    håndterFavorittKlikk,
    favoritter,
    størrelse,
}: KameraPanelProps) => {
    const stortBildeUrl = (kameraId: number) => `${WEBKAMERA_URL}/public/kamera?id=${kameraId}`;
    const [modalOpen, setModalOpen] = useState<boolean>(false);
    const { windowWidth } = useWindowSize();

    return (
        <div>
            <Panel $størrelse={størrelse} $maksbredde={windowWidth}>
                <Banner>
                    <BannerWrapper>
                        <Heading as="h2" $stylelevel="6">
                            {veg} {stedsnavn}
                        </Heading>
                        <Info>{info}</Info>
                    </BannerWrapper>
                    <KnappUtenStyling
                        onClick={() =>
                            håndterFavorittKlikk(
                                id.toString(),
                                veg + " " + stedsnavn,
                                favoritter?.includes(id.toString()) || false,
                            )
                        }
                    >
                        <FavorittIkon $favoritt={favoritter?.includes(id.toString())} />
                    </KnappUtenStyling>
                </Banner>

                <BildeLenke
                    $størrelse={størrelse}
                    onClick={() => {
                        setModalOpen(true);
                        loggEventMedTabellnavnOgVerdi(
                            TabellNavn.webkamera,
                            WebkameraLoggEvent.kameraModal,
                            stedsnavn + " " + veg,
                        );
                    }}
                >
                    <VegkameraBilde src={stortBildeUrl(id)} alt={`Vegkamera ${veg} ${stedsnavn}`} />
                </BildeLenke>
                <EksternLenke href={vaervarsel}>Åpne værstasjon i Yr</EksternLenke>
            </Panel>

            {modalOpen && (
                <WebkameraModal
                    onClose={() => setModalOpen(false)}
                    bildeUrl={stortBildeUrl(id)}
                    veg={veg}
                    stedsnavn={stedsnavn}
                    info={info}
                    vaervarsel={vaervarsel}
                    bildebredde={windowWidth - 60}
                />
            )}
        </div>
    );
};

const Panel = styled.div<{ $størrelse: KameraStørrelse; $maksbredde: number }>`
    font-family: "LFT Etica", sans-serif;
    width: ${(props) =>
        props.$størrelse === KameraStørrelse.Liten
            ? "240px"
            : props.$størrelse === KameraStørrelse.Medium
              ? "360px"
              : "480px"};

    max-width: ${(props) => props.$maksbredde - 30}px;
`;

const Banner = styled.div`
    background: var(--blue-bg);
    padding: 12px;
    line-height: 18px;
    gap: 16px;
    display: flex;
    justify-content: space-between;
    border-radius: 8px 8px 0 0;
`;

const BannerWrapper = styled.div`
    min-height: 36px;
`;

const Info = styled.p`
    font-weight: 300;
    font-size: 12px;
    margin: 0;
`;

const BildeLenke = styled.button<{ $størrelse: KameraStørrelse }>`
    border: 0;
    background: black;
    min-height: ${(props) =>
        props.$størrelse === KameraStørrelse.Liten
            ? "180px"
            : props.$størrelse === KameraStørrelse.Medium
              ? "270px"
              : "360px"};
    width: 100%;
    cursor: pointer;
    padding: 0;

    @media (max-width: 490px) {
        min-height: 0;
    }
`;

const VegkameraBilde = styled.img`
    max-height: 100%;
    max-width: 100%;
`;

const FavorittIkon = styled.i<{ $favoritt?: boolean }>`
    width: 22px;
    height: 22px;
    background: url(${(props) =>
            props.$favoritt ? "/ikon/favoritt_valgt.svg" : "/ikon/favoritt.svg"})
        no-repeat;
    background-size: contain;
    flex-shrink: 0;

    &:hover {
        background-image: url("/ikon/favoritt_hover.svg");
        cursor: pointer;
        scale: 1.2;
    }
`;
